#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libintl.h>

#include "config.h"
#include "gencfs.h"
#include "encrypteddirs.h"


void define_encrypteddirs_model () {
	GtkTreeView *view;
	GtkListStore *model;
	
	view = (GtkTreeView *) gtk_builder_get_object (widgetstree, "encrypteddirs");
	model = gtk_list_store_new (COLUMNS_COUNT,
		G_TYPE_STRING, /* encrypted directory */
		G_TYPE_STRING, /* mountpoint */
		G_TYPE_BOOLEAN); /* connected */
	gtk_tree_view_set_model (view, (GtkTreeModel *) model);
}


gchar **load_mounts () {
	gchar **mounts;
	gchar *output;
	gchar *delimiter = "\n";

	g_spawn_command_line_sync ("mount", &output, NULL, NULL, NULL);
        mounts = g_strsplit (output, delimiter, 0);
        g_free (output);
       
        return mounts;
}


gboolean is_mounted (gchar *mountpoint, gchar **mounts) {
	gboolean mounted;
	gint i;
     
	mounted = FALSE;
	i=0;
	while (i < g_strv_length (mounts) && ! mounted) {
		if (strlen (mountpoint) != 0 && g_ascii_strncasecmp (mountpoint, mounts[i] + 9, strlen (mountpoint)) == 0) {
			mounted = TRUE;
		}
		i++;
	}
     
	return mounted;
}


void load_encrypteddirs () {
	gchar *configfilepath;
	FILE *configfile;
	gchar line[201], *encrypteddir, *mountpoint;
	gchar **mounts;
	gint i;
	GtkTreeIter iter;
	GtkListStore *model;
	GtkTreeView *view;
	
	view = (GtkTreeView *) gtk_builder_get_object (widgetstree, "encrypteddirs");
	model = (GtkListStore *) gtk_tree_view_get_model (view);
	
	configfilepath = g_strdup_printf ("%s/%s", getenv ("HOME"), CONFIGFILE);
	
	configfile = fopen (configfilepath, "r");
	g_free (configfilepath);
	
	if (configfile != NULL) {
		mounts = load_mounts ();
		
		fgets (line, sizeof(line), configfile);
		while (! feof (configfile)) {
			i = 0; encrypteddir = line;
			while (line [i] != ':') i++; line[i] = '\0'; mountpoint = line + i + 1;
			while (line [i] != '\n') i++; line[i] = '\0';
			
			gtk_list_store_append (model, &iter);
			gtk_list_store_set (model, &iter,
				ENCRYPTEDDIR, encrypteddir,
				MOUNTPOINT, mountpoint,
				MOUNTED, is_mounted (mountpoint, mounts), -1);
			fgets (line, sizeof(line), configfile);
		}
		fclose (configfile);
		
		g_strfreev (mounts);
	}
}


void save_encrypteddirs () {
	gchar *configfilepath;
	FILE *configfile;
	gchar *encrypteddir, *mountpoint;
	GtkTreeIter iter;
	GtkListStore *model;
	GtkTreeView *view;
	gboolean available_iter; 
	
	configfilepath = g_strdup_printf ("%s/%s", getenv ("HOME"), CONFIGFILE);
	configfile = fopen (configfilepath, "w");
	g_free (configfilepath);
	
	if (configfile != NULL) {
		view = (GtkTreeView *) gtk_builder_get_object (widgetstree, "encrypteddirs");
		model = (GtkListStore *) gtk_tree_view_get_model (view);
		available_iter = gtk_tree_model_get_iter_first ((GtkTreeModel *) model, &iter);
		while (TRUE == available_iter) {
			gtk_tree_model_get ((GtkTreeModel *) model, &iter, ENCRYPTEDDIR, &encrypteddir, MOUNTPOINT, &mountpoint, -1);
			fprintf (configfile, "%s:%s\n", encrypteddir, mountpoint);
			g_free (encrypteddir);
			g_free (mountpoint);
			available_iter = gtk_tree_model_iter_next ((GtkTreeModel *) model, &iter);
		}
		fclose (configfile);
	}
	edited = FALSE;
}


gboolean on_encrypteddir_change (
	GtkTreeSelection	*selection,
	GtkTreeModel		*model,
        GtkTreePath		*path,
	gboolean			path_currently_selected,
	gpointer			userdata)
{
	GtkTreeIter iter;
	gboolean mounted;
	GtkButton *remove, *mount, *umount, *choosefolder;
	
	if (! path_currently_selected) {
		gtk_tree_model_get_iter (model, &iter, path);
		gtk_tree_model_get (model, &iter, MOUNTED, &mounted, -1);
		remove = (GtkButton *) gtk_builder_get_object (widgetstree, "remove");
		mount = (GtkButton *) gtk_builder_get_object (widgetstree, "mount");
		umount = (GtkButton *) gtk_builder_get_object (widgetstree, "umount");
		choosefolder = (GtkButton *) gtk_builder_get_object (widgetstree, "choosefolder");
		if (mounted) {
			gtk_widget_set_sensitive ((GtkWidget *) remove, FALSE);
			gtk_widget_set_sensitive ((GtkWidget *) mount, FALSE);
			gtk_widget_set_sensitive ((GtkWidget *) umount, TRUE);
			gtk_widget_set_sensitive ((GtkWidget *) choosefolder, FALSE);
		} else {
			gtk_widget_set_sensitive ((GtkWidget *) remove, TRUE);
			gtk_widget_set_sensitive ((GtkWidget *) mount, TRUE);
			gtk_widget_set_sensitive ((GtkWidget *) umount, FALSE);
			gtk_widget_set_sensitive ((GtkWidget *) choosefolder, TRUE);
			}
	}
	return TRUE;
}


void add_encrypteddir (GtkWidget *widget, gpointer user_data) {
	GtkTreeIter iter;
	GtkTreeView *view;
	GtkListStore *model;
	
	view = (GtkTreeView *) gtk_builder_get_object (widgetstree, "encrypteddirs");
	model = (GtkListStore *) gtk_tree_view_get_model (view);
	gtk_list_store_append (model, &iter);
	gtk_list_store_set (model, &iter, ENCRYPTEDDIR, g_getenv ("HOME"), MOUNTPOINT, "", MOUNTED, FALSE, -1);
	edited = TRUE;
}


void remove_encrypteddir (GtkWidget *widget, gpointer user_data) {
	GtkTreeIter iter;
	GtkTreeSelection *selection;
	GtkTreeView *view;
	GtkListStore *model;
	
	view = (GtkTreeView *) gtk_builder_get_object (widgetstree, "encrypteddirs");
	selection = gtk_tree_view_get_selection (view);
	gtk_tree_selection_get_selected (selection, (GtkTreeModel **) &model, &iter);
	gtk_list_store_remove (model, &iter);
	
	gtk_widget_set_sensitive ((GtkWidget *) gtk_builder_get_object (widgetstree, "remove"), FALSE);
	gtk_widget_set_sensitive ((GtkWidget *) gtk_builder_get_object (widgetstree, "mount"), FALSE);
	gtk_widget_set_sensitive ((GtkWidget *) gtk_builder_get_object (widgetstree, "umount"), FALSE);
	gtk_widget_set_sensitive ((GtkWidget *) gtk_builder_get_object (widgetstree, "choosefolder"), FALSE);
	
	edited = TRUE;
}


void on_cell_edited (
	GtkCellRendererText	*cell,
	gchar				*path_string,
	gchar				*new_text,
	gpointer				user_data)
{
	GtkTreeIter iter;
	GtkListStore *model;
	GtkTreeView *view;
	gint column;
	gchar *mountpoint, *encrypteddir, *encrypteddirbasename;
	gboolean mounted;
	
	column = GPOINTER_TO_INT (user_data);
	
	view = (GtkTreeView *) gtk_builder_get_object (widgetstree, "encrypteddirs");
	model = (GtkListStore *) gtk_tree_view_get_model (view);
	gtk_tree_model_get_iter_from_string ((GtkTreeModel *) model, &iter, path_string);
	gtk_tree_model_get ((GtkTreeModel *) model, &iter, MOUNTED, &mounted, -1);
	
	if (mounted) {
		umount_encrypteddir(NULL, NULL);
		gtk_tree_model_get ((GtkTreeModel *) model, &iter, MOUNTED, &mounted, -1);
	}
	
	if (! mounted) {
		gtk_list_store_set (model, &iter, column, new_text, -1);
		
		encrypteddir = new_text;
		encrypteddirbasename = g_path_get_basename (encrypteddir);
		mountpoint = g_strdup_printf ("%s/%s", encrypteddir, encrypteddirbasename);
		
		if (ENCRYPTEDDIR == column) {
			gtk_list_store_set (model, &iter, MOUNTPOINT, mountpoint, -1);
		}
		
		g_free (mountpoint);
		g_free (encrypteddirbasename);
		
		edited = TRUE;
	}
}


void mount_encrypteddir (GtkWidget *widget, gpointer user_data) {
	GtkTreeIter iter;
	GtkTreeSelection *selection;
	GtkTreeView *view;
	GtkListStore *model;
	gchar *encrypteddir, *mountpoint;
	gchar *command, **encfscommand, *encfscommandline;
	gboolean success;
	GPid encfspid;
	GtkTreePath * path;
	
	view = (GtkTreeView *) gtk_builder_get_object (widgetstree, "encrypteddirs");
	selection = gtk_tree_view_get_selection (view);
	gtk_tree_selection_get_selected (selection, (GtkTreeModel **) &model, &iter);
	gtk_tree_model_get ((GtkTreeModel *) model, &iter, ENCRYPTEDDIR, &encrypteddir, MOUNTPOINT, &mountpoint, -1);
	
	if (0 == strlen (encrypteddir) || 0 == strlen (mountpoint))
		display_error (gettext ("Error: please fill encrypted directory and mountpoint."));
	else {
		command = g_strdup_printf ("mkdir -p '%s'", mountpoint);
		success = g_spawn_command_line_sync (command, NULL, NULL, NULL, NULL);
		g_free (command);
		if (! success)
			display_error (gettext ("Error: mount point can't be created."));
		else {
			path = gtk_tree_model_get_path ((GtkTreeModel *) model, &iter);
			encfscommandline = g_strdup_printf ("setsid encfs -o nonempty --extpass gencfs-askpass \"%s\" \"%s\"", encrypteddir, mountpoint);
			g_shell_parse_argv (encfscommandline, NULL, &encfscommand, NULL);
			g_free (encfscommandline);
			g_spawn_async (NULL, encfscommand, NULL, 
				G_SPAWN_SEARCH_PATH|G_SPAWN_DO_NOT_REAP_CHILD, 
				NULL, NULL, &encfspid, NULL);
			g_child_watch_add (encfspid, (GChildWatchFunc) encfscommandfinished, (gpointer) path);
			g_strfreev (encfscommand);
		}
	}
	
	g_free (encrypteddir);
	g_free (mountpoint);
}


void umount_encrypteddir (GtkWidget *widget, gpointer user_data) {
	GtkTreeIter iter;
	GtkTreeSelection *selection;
	GtkTreeView *view;
	GtkListStore *model;
	gchar *encrypteddir, *mountpoint;
	gchar *command;
	gchar **mounts;
	
	view = (GtkTreeView *) gtk_builder_get_object (widgetstree, "encrypteddirs");
	selection = gtk_tree_view_get_selection (view);
	gtk_tree_selection_get_selected (selection, (GtkTreeModel **) &model, &iter);
	gtk_tree_model_get ((GtkTreeModel *) model, &iter, ENCRYPTEDDIR, &encrypteddir, MOUNTPOINT, &mountpoint, -1);
	
	command = g_strdup_printf ("fusermount -u '%s'", mountpoint);
	g_spawn_command_line_sync (command, NULL, NULL, NULL, NULL);
	g_free (command);
	
	mounts = load_mounts();
	if (! is_mounted (mountpoint, mounts)) {
		gtk_list_store_set (model, &iter, MOUNTED, FALSE, -1);
		gtk_widget_set_sensitive ((GtkWidget *) gtk_builder_get_object (widgetstree, "remove"), TRUE);
		gtk_widget_set_sensitive ((GtkWidget *) gtk_builder_get_object (widgetstree, "mount"), TRUE);
		gtk_widget_set_sensitive ((GtkWidget *) gtk_builder_get_object (widgetstree, "umount"), FALSE);
		gtk_widget_set_sensitive ((GtkWidget *) gtk_builder_get_object (widgetstree, "choosefolder"), TRUE);
		
		command = g_strdup_printf ("rmdir '%s'", mountpoint);
		g_spawn_command_line_sync (command, NULL, NULL, NULL, NULL);
		g_free (command);
	} else {
		display_error (gettext ("Error: mount point is occupied."));
	}
	g_strfreev (mounts);
	
	g_free (encrypteddir);
	g_free (mountpoint);
}


void encfscommandfinished (GPid pid, gint status, gpointer data) {
	GtkTreePath *path = (GtkTreePath *) data;
	gchar **mounts;
	GtkTreeIter iter;
	GtkTreeView *view;
	GtkListStore *model;
	gchar *encrypteddir, *mountpoint;
	gchar *command;
	
	g_print ("DEBUG STATUS=%d\n", status);
	
	mounts = load_mounts();
	view = (GtkTreeView *) gtk_builder_get_object (widgetstree, "encrypteddirs");
	model = (GtkListStore *) gtk_tree_view_get_model (view);
	gtk_tree_model_get_iter ((GtkTreeModel *) model, &iter, path);
	gtk_tree_model_get ((GtkTreeModel *) model, &iter, ENCRYPTEDDIR, &encrypteddir, MOUNTPOINT, &mountpoint, -1);
	
	if (is_mounted (mountpoint, mounts)) {
		gtk_list_store_set (model, &iter, MOUNTED, TRUE, -1);
		gtk_widget_set_sensitive ((GtkWidget *) gtk_builder_get_object (widgetstree, "remove"), FALSE);
		gtk_widget_set_sensitive ((GtkWidget *) gtk_builder_get_object (widgetstree, "mount"), FALSE);
		gtk_widget_set_sensitive ((GtkWidget *) gtk_builder_get_object (widgetstree, "umount"), TRUE);
		gtk_widget_set_sensitive ((GtkWidget *) gtk_builder_get_object (widgetstree, "choosefolder"), FALSE);
		
		command = g_strdup_printf ("xdg-open '%s'", mountpoint);
		g_spawn_command_line_sync (command, NULL, NULL, NULL, NULL);
		g_free (command);
		
		save_encrypteddirs();
	} else {
		command = g_strdup_printf ("rmdir '%s'", mountpoint);
		g_spawn_command_line_sync (command, NULL, NULL, NULL, NULL);
		g_free (command);
		display_error (gettext ("Error: cancelled or invalid password."));
	}
	
	g_strfreev (mounts);
	g_free (encrypteddir);
	g_free (mountpoint);
	gtk_tree_path_free (path);
}


void set_encrypted_directory (GtkWidget *widget, gpointer user_data) {
	GtkTreeIter iter;
	GtkListStore *model;
	GtkTreeSelection *selection;
	GtkTreeView *view;
	GtkFileChooserDialog *choosefolderdialog;
	gint column;
	gchar *mountpoint, *encrypteddir, *encrypteddirbasename;
	
	column = GPOINTER_TO_INT (user_data);
	
	view = (GtkTreeView *) gtk_builder_get_object (widgetstree, "encrypteddirs");
	choosefolderdialog = (GtkFileChooserDialog *) gtk_builder_get_object (widgetstree, "choosefolderdialog");
	selection = gtk_tree_view_get_selection (view);
	gtk_tree_selection_get_selected (selection, (GtkTreeModel **) &model, &iter);
	
	encrypteddir = gtk_file_chooser_get_filename ((GtkFileChooser *) choosefolderdialog);
	encrypteddirbasename = g_path_get_basename (encrypteddir);
	mountpoint = g_strdup_printf ("%s/%s", encrypteddir, encrypteddirbasename);
	gtk_list_store_set (model, &iter, ENCRYPTEDDIR, encrypteddir, MOUNTPOINT, mountpoint, -1);
	g_free (encrypteddir);
	g_free (mountpoint);
	g_free (encrypteddirbasename);
	
	edited = TRUE;
}
